=== modified file 'bzr_man.py'
--- bzr_man.py	2005-12-10 00:00:50 +0000
+++ bzr_man.py	2005-12-10 01:29:30 +0000
@@ -18,119 +18,59 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os, sys
-import bzrlib, bzrlib.help
+import bzrlib, bzrlib.help, bzrlib.commands
 #<<< Begin (C) Hans Ulrich Niedermann
 import time
 import re
 
+def main():
+    t = time.time()
+    tt = time.gmtime(t)
+    params = \
+           { "bzrcmd": "bzr",
+             "datestamp": time.strftime("%Y-%m-%d",tt),
+             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
+             "version": bzrlib.__version__,
+             }
+
+    filename = "bzr.1"
+    if len(sys.argv) == 2:
+        filename = sys.argv[1]
+    if filename == "-":
+        outfile = sys.stdout
+    else:
+        outfile = open(filename,"w")
+
+    outfile.write(man_preamble % params)
+    outfile.write(man_escape(man_head % params))
+#<<< End (C) Hans Ulrich Niedermann
+    outfile.write(man_escape(getcommand_help()))
+#<<< Begin (C) Hans Ulrich Niedermann
+    outfile.write(man_escape(man_foot % params))
+
 def man_escape(string):
     result = string.replace("\\","\\\\")
     result = result.replace("`","\\`")
     result = result.replace("'","\\'")
     result = result.replace("-","\\-")
     return result
-
-
-class Parser:
-
-    def parse_line(self, line):
-        pass
-
-
-class CommandListParser(Parser):
-
-    """Parser for output of "bzr help commands".
-
-    The parsed content can then be used to
-    - write a "COMMAND OVERVIEW" section into a man page
-    - provide a list of all commands
-    """
-
-    def __init__(self,params):
-        self.params = params
-        self.command_usage = []
-        self.all_commands = []
-        self.usage_exp = re.compile("([a-z0-9-]+).*")
-        self.descr_exp = re.compile("    ([A-Z].*)\s*")
-        self.state = 0
-        self.command = None
-        self.usage = None
-        self.descr = None
-
-    def parse_line(self, line):
-        m = self.usage_exp.match(line)
-	if line == '':
-		return
-        if m:
-            if self.state == 0:
-                if self.usage:
-                    self.command_usage.append((self.command,self.usage,self.descr))
-                    self.all_commands.append(self.command)
-                self.usage = " ".join(line.split(" ")[1:])
-                self.command = m.groups()[0]
-            else:
-                raise RuntimeError, "matching usage line in state %d" % state
-            self.state = 1
-            return
-        m = self.descr_exp.match(line)
-        if m:
-            if self.state == 1:
-                self.descr = m.groups()[0]
-            else:
-                raise RuntimeError, "matching descr line in state %d" % state
-            self.state = 0
-            return
-        raise RuntimeError, "Cannot parse this line ('%s')." % line
-
-    def end_parse(self):
-        if self.state == 0:
-            if self.usage:
-                self.command_usage.append((self.command,self.usage,self.descr))
-                self.all_commands.append(self.command)
-        else:
-            raise RuntimeError, "ending parse in state %d" % state
-
-    def write_to_manpage(self, outfile):
-        bzrcmd = self.params["bzrcmd"]
-        outfile.write('.SH "COMMAND OVERVIEW"\n')
-        for (command,usage,descr) in self.command_usage:
-            outfile.write('.TP\n.B "%s %s"\n%s\n' % (bzrcmd, usage, descr))
-
-
-class HelpReader:
-
-    def __init__(self, parser):
-        self.parser = parser
-
-    def write(self, data):
-        if data[-1] == '\n':
-            data = data[:-1]
-        for line in data.split('\n'):
-            self.parser.parse_line(line)
-
-
-def write_command_details(params, command, usage, descr, outfile):
-    x = ('.SS "%s %s"\n.B "%s"\n.PP\n.B "Usage:"\n%s %s\n\n' %
-         (params["bzrcmd"],
-          command,
-          descr,
-          params["bzrcmd"],
-          usage))
-    outfile.write(man_escape(x))
-
-
-man_preamble = """\
-.\\\" Man page for %(bzrcmd)s (bazaar-ng)
-.\\\"
-.\\\" Large parts of this file are autogenerated from the output of
-.\\\"     \"%(bzrcmd)s help commands\"
-.\\\"     \"%(bzrcmd)s help <cmd>\"
-.\\\"
-.\\\" Generation time: %(timestamp)s
-.\\\"
-"""
 #<<< End (C) Hans Ulrich Niedermann
 
+def format_command (cmd):
+    cmdusage = '.TP\n.B %s\n.PP\n' % (bzrlib.help.command_usage(cmd))
+    doc = "%s\n.PP\n.PP\n" % (cmd.__doc__)
+    return cmdusage + doc
+
+def getcommand_help():
+    output=""
+    command_names = bzrlib.commands.builtin_command_names()
+    command_names.sort()
+    for cmd_name in  command_names:
+        desc = bzrlib.commands.get_cmd_object(cmd_name)
+        output = output + format_command(desc)
+    return output
+
+
 man_head = """\
 .TH bzr 1 "%(datestamp)s" "%(version)s" "bazaar-ng"
 .SH "NAME"
@@ -211,40 +151,16 @@
 """
 
 #<<< Begin (C) Hans Ulrich Niedermann
-def main():
-    t = time.time()
-    tt = time.gmtime(t)
-    params = \
-           { "bzrcmd": "bzr",
-             "datestamp": time.strftime("%Y-%m-%d",tt),
-             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
-             "version": bzrlib.__version__,
-             }
-
-    clp = CommandListParser(params)
-    bzrlib.help.help("commands", outfile=HelpReader(clp))
-    clp.end_parse()
-
-    filename = "bzr.1"
-    if len(sys.argv) == 2:
-        filename = sys.argv[1]
-    if filename == "-":
-        outfile = sys.stdout
-    else:
-        outfile = open(filename,"w")
-
-    outfile.write(man_preamble % params)
-    outfile.write(man_escape(man_head % params))
-    clp.write_to_manpage(outfile)
-
-    # FIXME:
-    #   This doesn't do more than the summary so far.
-    #outfile.write('.SH "DETAILED COMMAND DESCRIPTION"\n')
-    #for (command,usage,descr) in clp.command_usage:
-    #    write_command_details(params, command, usage, descr, outfile = outfile)
-
-    outfile.write(man_escape(man_foot % params))
-
+man_preamble = """\
+.\\\" Man page for %(bzrcmd)s (bazaar-ng)
+.\\\"
+.\\\" Large parts of this file are autogenerated from the output of
+.\\\"     \"%(bzrcmd)s help commands\"
+.\\\"     \"%(bzrcmd)s help <cmd>\"
+.\\\"
+.\\\" Generation time: %(timestamp)s
+.\\\"
+"""
 
 if __name__ == '__main__':
     main()

